// original source in Automation guide
//- 0 - All tests passed successfully
//- 1 - Some tests failed
//- 3 - No supported browsers available on this machine
//- 4 - No tests to run (probably filter doesn't match any test url)
//- 5 - Can't open harness page
//- 6 - Wrong command line arguments
//- 8 - Exit after showing the Siesta version (when `--version` is provided on the command line)
//- 9 - java is not installed or not available in PATH

var fs          = require('fs')
var helper      = require('./lib/helper.js')
var chai        = require('chai')

var expect      = chai.expect

describe('Exit codes', function() {
    this.timeout(90000)
    
    it('For failed test suite', function (done) {
        
        helper.runSuite({ $harnessUrl : 'http://lh/siesta/tests/launchers/suites/failing_suite' }, function (res) {
            expect(res.exitCode).to.equal(1)
            done()
        })
    });
    
    it('For `no descriptors` to run', function (done) {
        
        helper.runSuite({ 'include' : 'quix' }, function (res) {
            expect(res.exitCode).to.equal(4)
            done()
        })
    });
    
    it('For wrong harness url', function (done) {
        
        helper.runSuite({ $harnessUrl : 'quix' }, function (res) {
            expect(res.exitCode).to.equal(5)
            done()
        })
    });
    
    it('For wrong command line options', function (done) {
        
        helper.runSuite({ 'report-format' : 'quix' }, function (res) {
            expect(res.exitCode).to.equal(6)
            done()
        })
    });
    
    it('For --version switch', function (done) {
        
        helper.runSuite({ 'version' : true }, function (res) {
            expect(res.exitCode).to.equal(8)
            done()
        })
    });
});